#!/usr/bin/perl -T

# post.cgi: Writes to file so that comment.cgi can process it.
# Copyright (C) <2021>  <Jake Thoughts>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# Author       : Jake Thoughts jake@jakes-mail.top
# Release Date : 2021 Feb 03
# Version      : 0.8
# TODO         : captcha implimentation.

use strict;
use warnings;
use Time::Piece;
use utf8;

local (our $buffer, our @pairs, our $pair, our $name, our $value, our %FORM);

$ENV{'REQUEST_METHOD'} =~ tr/a-z/A-Z/;
if ($ENV{'REQUEST_METHOD'} eq "POST") {
   read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
}

@pairs = split(/&/, $buffer);
foreach $pair (@pairs) {
   ($name, $value) = split(/=/, $pair);
   $value =~ tr/+/ /;
   $value =~ s/%([a-fA-F0-9] [a-fA-F0-9])/pack("C", hex($1))/eg;
   $FORM{$name} = $value;
}
my $username		= $FORM{username};
my $email			= $FORM{email};
my $comment			= $FORM{comment};
my $captcha			= $FORM{captcha};
my $date			= localtime->strftime('%Y-%m-%d,%R:%S');
my $blog			= $FORM{blog};
my $name_file		= "../protected/names.txt";
my $html_posttop	= './html/post.html'; # no need for a navigation bar when redirection is involved
my $html_bottom 	= './html/bottom.html';
my $bad_reload_time	= 9999;
my $ok_reload_time	= 1;

if ($blog =~ /^([-\w.]+)$/ ) { # blog is tainted and needs to be untainted since a file "$blog" gets written to
	$blog = $1;
	unless (-e "../protected/comments/$blog.txt") { # makes sure comment file for the blog page actually exist. 
		die "Could not access $blog.txt";	
	}
} else {
	$blog = "";
	die "$date post.cgi !!!Variable blog did not match!!! $ENV{'REMOTE_ADDR'}";
}

sub create_upper_html{
	print "Content-type:text/html\r\n\r\n";
	open(FH, "<", $html_posttop) or die "post.cgi Cannot open $html_posttop";
		while(<FH>){
			print "$_";
		}
	close(FH);
}

sub create_bottom_html{
	open(FH, "<", $html_bottom) or die "post.cgi Cannot open $html_bottom";
		while(<FH>){
			print "$_";
		}
	close(FH);
}

if ($username eq ""){ # username can be set or selected automatically from a list
	open(my $FH, '<', $name_file) or die "post.cgi Can't open $name_file";
	my @names;
	while (my $name = <$FH> ) {
		push (@names, $name);
	}
	close($FH) or die "cannot close";
	my $names = $names[ rand @names ];
	$username = $names;
	$username =~ s/\n//;
}

if ($comment eq ""){ # comment is required but easily defeated
	$comment = "Nice try :^)";
}

if (lc $captcha ne "thoughts"){
	create_upper_html();
	print "<div class='block'>";
	if ($blog eq "guest") {
		print "<meta http-equiv=\"Refresh\" content=\"$bad_reload_time; url='guest.html'\" />";
	} else {	
		print "<meta http-equiv=\"Refresh\" content=\"$bad_reload_time; url='blog/$blog.shtml'\" />";
	}
	print "<p>Greetings, $username, you failed the captcha. You haven't <b>thoughts</b> this out have you? Press back arrow on browser... usually this brings your stuff back.</p>";
	print "</div>";
	create_bottom_html();
	exit;
}

if ((length $username >= 90) or (length $email >= 160) or (length $comment >= 12000)) {
	create_upper_html();
	print "<div class='block'>";
	if ($blog eq "guest") {
		print "<meta http-equiv=\"Refresh\" content=\"$bad_reload_time; url='guest.html'\" />";
	} else {	
		print "<meta http-equiv=\"Refresh\" content=\"$bad_reload_time; url='blog/$blog.shtml'\" />";
	}
	print "<p>System error; check length of inputs. Username 90, email 160, comment 12000. Press back arrow on browser... usually this brings your stuff back.</p>";
	print "</div>";
	create_bottom_html();
	exit;
}

open(FH, '>>', "../protected/comments/$blog.txt") or die "post.cgi Cannot open ../protected/comments/$blog.txt";
if ($email eq "") {
	print FH "<div class=\"guest_info\"><span>$username</span> on $date said:</div><p>$comment</p>\n";
} else { 
	print FH "<div class=\"guest_info\"><a href=\"mailto:$email\"><span>$username</span></a> on $date said:</div><p>$comment</p>\n";
}
close(FH);

open(FH, '>>', "../protected/comments/all_comments.txt") or die "Cannot open ../protected/comments/all_comments.txt";
if ($email eq "") {
	print FH "$blog <div class=\"guest_info\"><span>$username</span> on $date said:</div><p>$comment</p>\n";
} else { 
	print FH "$blog <div class=\"guest_info\"><a href=\"mailto:$email\"><span>$username</span></a> on $date said:</div><p>$comment</p>\n";
}
close(FH);

create_upper_html();
print "<div class='block'>";
if ($blog eq "guest") {
	print "<meta http-equiv=\"Refresh\" content=\"$ok_reload_time; url='guest.html'\" />";
} else {	
	print "<meta http-equiv=\"Refresh\" content=\"$ok_reload_time; url='blog/$blog.shtml'\" />";
}
print "<p>Greetings, $username, please wait while your comment gets posted!</p>";
print "</div>";
create_bottom_html();

1;
